#ifndef _CONTACTLIST_H

#define _CONTACTLIST_H

#include <gtk/gtk.h>
#include "imuserdaemon.h"
#include "gtkcellrenderertextpixbuf.h"
#include "userutilities.h"
#include "icons.h"
#include "settings.h"

enum ContactType
{
	CE_UNDEFINED,
	CE_GROUP,
	CE_PROTO,
	CE_USER,
	CE_MULTUSER
};


enum ContactStoreField
{
	CL_SELECTED,
	CL_ICON,
	CL_ICONVISIBLE,
	CL_ALIAS,
	CL_ORDER,
	CL_BGCOLOR,
	CL_BGCOLORACTIVE,
	CL_STATEICONS,
	CL_TEXTATTR,
	CL_ENTRYPOINTER
};


enum ContactFlashType
{
	CF_WEIGHT = 1,
	CF_STYLE = 2,
	CF_COLOR = 4,
	CF_LETTERS = 8
};
	

enum ContactSortType
{
	CS_STATUS,
	CS_LASTRECEIVED,
	CS_LASTSENT,
	CS_ALPHABETICAL,
	CS_IDLE,
	CS_LASTSESSIONEVENT
};


// identifers for the contact list view modes
enum ContactViewType
{
	CVIEW_GROUPS = 1,
	CVIEW_ONEGROUP = 2,
	CVIEW_NOEMPTYGROUPS = 4,
	CVIEW_NOGROUPS = 8,
	CVIEW_HIDEOFFLINE = 16,
	CVIEW_HIDEOFFLINEGROUPS = 32,
	CVIEW_SHOWCHECKBOX = 64
};


class IMContactListUser;
class IMContactListGroup;
class IMContactList;


class IMContactListEntry
{
	public:
		IMContactListEntry(gchar* _id, gpointer _data);
		virtual ~IMContactListEntry();
		
		// functions for child handling
		virtual void 	addChild(IMContactListEntry* child);
		virtual void 	removeChild(IMContactListEntry* child);
		void			deleteAllChildren();
		gboolean 		hasChildren();
		virtual void	childUpdated(IMContactListEntry* child);
		
		// querying children
		IMContactListEntry  *findChildByID(gchar* _id, ContactType cType);
		IMContactListEntry  *getParent();
		GList			  *getAllChildrenOfType(ContactType toFind, GList* startList = NULL);
		GList			  *getChildren();	
	
		// display functions
		virtual void		realizeEntry(gboolean realizeChildren, gboolean realizeThisEntry = TRUE);
		virtual void		unrealizeEntry();
		virtual void		updateEntry(gboolean updateChildren);
		virtual void		updateParents();
		void				setViewType(ContactViewType viewType);
		virtual gboolean	canBeVisible();
		virtual void		updateColors();
	
		// misc functions
		virtual guint		getNumOnline();
		guint				getNumRecursiveChildren();
		void				setViewMode(ContactViewType _viewType);
		void				setTreeModel(GtkTreeModel* _model);
		
		// selection functions
		virtual void		setSelected(gboolean setting);
		gboolean			getSelected();
		GList				*getSelectedChildren(ContactType toFind, GList* startList = NULL);
						
		ContactType 		type;
		ContactViewType		viewType;
		gpointer 			data;
		GtkWidget			*widget;
		gchar	 			*id;		
		
		
	private:
		GtkTreeIter			*iter;
		GtkTreeModel		*model;
		GList				*children;
		IMContactListEntry	*parent;
		gboolean			selected;
	
	friend class IMContactListUser;
	friend class IMContactListGroup;
	friend class IMContactList;
};


class IMContactListGroup: public IMContactListEntry
{
	public:
		IMContactListGroup(gchar* _name, gchar* _id, gpointer _data);
		~IMContactListGroup();
		
		// open / close group operations
		void 	 openGroup(gboolean open);
		gboolean isGroupOpen();
	
		// querying children
		void	removeChild(IMContactListEntry* child);
		void	childUpdated(IMContactListEntry* child);
		
		// display functions
		void	 updateEntry(gboolean updateChildren);
		gboolean canBeVisible();
		void	 updateColors();
	
		void setVisible(gboolean _visible);
	
	private:
		gchar			*name;
		gboolean		isOpen,
						arrowSelected;
		PangoAttrList	*textAttributes;	
		GdkPixbuf		*arrow;
		gboolean		visible;
		GtkWidget		*quickMenuEntry;
	
	friend class IMContactList;
	friend class IMContactListUser;
	friend class IMContactListEntry;
};



class IMContactListUser: public IMContactListEntry
{
	public:	
		GList				*shadows;
	
		IMContactListUser(IMUserDaemon *user, GList *_shadows);
		~IMContactListUser();
		
		// display functions
		void	updateEntry(gboolean updateChildren);
		void	updateColors();
		void	setStatus(guint _status, GdkPixbuf *_icon);	
		gboolean canBeVisible();
		void	setSelected(gboolean setting);
	
		// blinking functions
		void	blinkIcon(GdkPixbuf *_iconBlink);	
		void	flashName(ContactFlashType ftType);
		void	stopFlashName();
		void	setFlashColor(GdkColor col);
	
		guint	getNumOnline();
	
		// color functions
		void		setBackgroundColor(GdkColor bgColor);
		void		setBackgroundColorEnabled(gboolean setting);
		gboolean	getBackgroundColorEnabled();
		void		setForegroundColor(GdkColor fgColor);
		void		setForegroundColorEnabled(gboolean setting);
		gboolean	getForegroundColorEnabled();
	
		// state icon functions
		void		setStateMask(gulong newMask);
	
	private:
		guint				status,
							blinkSource,
							flashSource;
		GdkPixbuf			*icon,
							*iconBlink;
		GdkColor			foregroundColor,
							backgroundColor,
							flashColor;
		gboolean			blinkTimeoutIcon,
							enableBgColor,
							enableFgColor;
		gulong				stateMask;
		PangoAttrList		*textAttributes;
		PangoAttribute		*textItalic,
							*textBold,
							*textColor;
		ContactFlashType	flashRules;
		guchar				flashIterator;
		guchar				flashLetters;
	
		// timeouts
		static gboolean cb_blinkIconTimeout(IMContactListUser* self);
		static gboolean cb_flashNameTimeout(IMContactListUser* self);
		
	friend class IMContactList;
};


class IMContactList: public IMContactListEntry
{
	public:
		IMContactList();
		~IMContactList();
		
		IMContactListEntry	*getFocusedEntry();
		GList				*getMultipleFocusedEntries();
		IMContactListEntry	*getEntryAtCursor(gint x, gint y);
		IMContactListEntry	*getEntryAtDropPosition(gint x, gint y);
		IMContactListEntry	*getContextEntry(GdkDragContext* dc);
		void				clearList();
		void				setCheckSelectionMode(gboolean setting);
		void				setShowExtendedIcons(gboolean setting);
		void				focusChild(IMContactListEntry* child);
	
		void				realizeEntry(gboolean realizeChildren);
	
		void				updateExtendedIconsList();
		void				updateViewMode();
	
		void				resortContactlist();
	
		void				setMultipleHighlight(gboolean setting);
		
		void				addChild(IMContactListEntry* child);
		
		void				toggleViewType(gint toToggle);
	
		GtkWidget		*frame,
						*quickBar;
						
		GtkWidget		*filterMenu,
						*filterLabel,
						*filterAllItem;
	
	private:
		GtkListStore		*listStore;
		GtkCellRenderer		*aliasRenderer;
		GList				*stateIconList;
		GtkTreePath			*oldPath;
		GtkTreeViewColumn	*column,
							*checkboxColumn;
		gboolean			useOwnFont;
		gchar				*font;
		GtkToolItem			*groupFilter;
	
	
		void createQuickBar();
	
		// callbacks
		static void cb_rowExpandedCollapsed(IMContactList* self, GtkTreeIter *eventIter, GtkTreePath *path);
		static gint cb_sortRowsFunction(GtkTreeModel* model, GtkTreeIter *iter1, GtkTreeIter *iter2, IMContactList* self);
		static void cb_checkboxToggled(GtkCellRenderer* t, gchar* pathString, IMContactList* self);

		static void cb_rowActivated(GtkTreeView *tv, GtkTreePath *path, GtkTreeViewColumn *column, IMContactList* self);

		static gboolean cb_viewMotionEvent(IMContactList* self, GdkEvent* event);
		static gboolean cb_rowClicked(IMContactList* self, GdkEventButton* event);
	
		static void cb_SettingsChangedCallback(gchar* sname, GList *values, IMContactList* self);
		
		static void cb_showOfflineButtonClicked(IMContactList* self);
		static void cb_showGroupsButtonClicked(IMContactList* self);
		static void cb_groupFilterButtonClicked(IMContactList* self);
		static void cb_groupAllFilterButtonClicked(IMContactList* self);
		
		static void cb_quickItemActivated(GtkMenuItem* item, IMContactList* self);
};

void IMContactList_updateSortRules();
#endif
